"""Models for operation events, returned by the MQTT broker.

Operation Event example:
{
    "version": 1,
    "traceId": "800a74737b5a4328862d958c35b71b74",
    "operation": "start-window-heating",
    "status": "ERROR",
    "errorCode": "timeout",
    "requestId": "5a16b265-85e7-4502-bd24-c92091c3df31",
}
"""

from dataclasses import dataclass, field
from enum import StrEnum

from mashumaro import field_options

from .base import BaseEvent, EventType


class OperationStatus(StrEnum):
    IN_PROGRESS = "IN_PROGRESS"
    COMPLETED_SUCCESS = "COMPLETED_SUCCESS"
    COMPLETED_WARNING = "COMPLETED_WARNING"
    ERROR = "ERROR"


class OperationName(StrEnum):
    APPLY_BACKUP = "apply-backup"
    LOCK = "lock"
    SET_AIR_CONDITIONING_AT_UNLOCK = "set-air-conditioning-at-unlock"
    SET_AIR_CONDITIONING_SEATS_HEATING = "set-air-conditioning-seats-heating"
    SET_AIR_CONDITIONING_TARGET_TEMPERATURE = "set-air-conditioning-target-temperature"
    SET_AIR_CONDITIONING_TIMERS = "set-air-conditioning-timers"
    SET_AIR_CONDITIONING_WITHOUT_EXTERNAL_POWER = "set-air-conditioning-without-external-power"
    SET_CLIMATE_PLANS = "set-climate-plans"
    START_ACTIVE_VENTILATION = "start-active-ventilation"
    START_AIR_CONDITIONING = "start-air-conditioning"
    START_AUXILIARY_HEATING = "start-auxiliary-heating"
    START_CHARGING = "start-charging"
    START_FLASH = "start-flash"
    START_HONK = "start-honk"
    START_WINDOW_HEATING = "start-window-heating"
    START_STOP_CHARGING = "start-stop-charging"
    STOP_ACTIVE_VENTILATION = "stop-active-ventilation"
    STOP_AIR_CONDITIONING = "stop-air-conditioning"
    STOP_AUXILIARY_HEATING = "stop-auxiliary-heating"
    STOP_CHARGING = "stop-charging"
    STOP_WINDOW_HEATING = "stop-window-heating"
    UNLOCK = "unlock"
    UPDATE_AUTO_UNLOCK_PLUG = "update-auto-unlock-plug"
    UPDATE_BATTERY_SUPPORT = "update-battery-support"
    UPDATE_CARE_MODE = "update-care-mode"
    UPDATE_CHARGE_LIMIT = "update-charge-limit"
    UPDATE_CHARGE_MODE = "update-charge-mode"
    UPDATE_CHARGING_CURRENT = "update-charging-current"
    UPDATE_CHARGING_PROFILES = "update-charging-profiles"
    UPDATE_DEPARTURE_TIMERS = "update-departure-timers"
    UPDATE_MINIMAL_SOC = "update-minimal-soc"
    UPDATE_TARGET_TEMPERATURE = "update-target-temperature"
    WAKEUP = "wakeup"
    WINDOWS_HEATING = "windows-heating"


@dataclass(frozen=True)
class OperationEvent(BaseEvent):
    """Base model for all Operation Events."""

    event_type = EventType.OPERATION
    request_id: str = field(metadata=field_options(alias="requestId"))
    operation: OperationName
    status: OperationStatus
    error_code: str | None = field(default=None, metadata=field_options(alias="errorCode"))


# TODO @dvx76: so also have a subclass per operation name here??



"""Unit tests for myskoda.models.service_event.

This module partially repeats test_mqtt.py tests but is more isolated:
here we have unit tests for service_event module only.
"""

from pathlib import Path

import pytest

from myskoda.models.charging import ChargeMode, ChargingState
from myskoda.models.event import (
    BaseEvent,
    ServiceEvent,
    ServiceEventChangeSoc,
    ServiceEventChangeSocData,
    ServiceEventChargingError,
    ServiceEventData,
    ServiceEventDepartureErrorPlug,
    ServiceEventError,
    ServiceEventErrorData,
)

FIXTURES_DIR = Path(__file__).parent.joinpath("fixtures")


@pytest.fixture(name="service_events")
def load_service_events() -> list[tuple[str, str]]:
    """Load service_events fixture. Return topic, payload."""
    vin = "TMOCKAA0AA000000"
    user_id = "b8bc126c-ee36-402b-8723-2c1c3dff8dec"
    topic_base = f"{user_id}/{vin}/service-event/"
    file_base = FIXTURES_DIR / "events"
    return [
        (
            f"{topic_base}charging",
            (file_base / "service_event_charging_change_soc.json").read_text(),
        ),
        (
            f"{topic_base}charging",
            (file_base / "service_event_charging_charging_status_changed.json").read_text(),
        ),
        (
            f"{topic_base}charging",
            (file_base / "service_event_charging_charging_error.json").read_text(),
        ),
        (
            f"{topic_base}departure",
            (file_base / "service_event_departure_ready.json").read_text(),
        ),
        (
            f"{topic_base}departure",
            (file_base / "service_event_departure_status_changed.json").read_text(),
        ),
        (
            f"{topic_base}departure",
            (file_base / "service_event_departure_error_plug.json").read_text(),
        ),
    ]


def test_parse_service_events(service_events: list[str]) -> None:
    for service_event in service_events:
        topic, payload = service_event
        event = BaseEvent.from_mqtt_message(topic=topic, payload=payload)

        assert isinstance(event, ServiceEvent)
        if isinstance(event, ServiceEventChangeSoc):
            assert event.data == ServiceEventChangeSocData(
                charged_range=195,
                mode=ChargeMode.MANUAL,
                soc=50,
                state=ChargingState.CHARGING,
                time_to_finish=440,
                user_id="ad0d7945-4814-43d0-801f-change-soc",
                vin="TMBAXXXXXXXXXXXXX",
            )
        elif isinstance(event, ServiceEventChargingError):
            assert event.data == ServiceEventErrorData(
                user_id=f"ad0d7945-4814-43d0-801f-{event.name.value}",
                vin="TMBAXXXXXXXXXXXXX",
                error_code=ServiceEventError.STOPPED_DEVICE,
            )
        elif isinstance(event, ServiceEventDepartureErrorPlug):
            assert event.data == ServiceEventErrorData(
                user_id=f"ad0d7945-4814-43d0-801f-{event.name.value}",
                vin="TMBAXXXXXXXXXXXXX",
                error_code=ServiceEventError.CLIMA,
            )
        else:
            assert event.data == ServiceEventData(
                user_id=f"ad0d7945-4814-43d0-801f-{event.name.value}",
                vin="TMBAXXXXXXXXXXXXX",
            )



"""Unit tests for myskoda.rest_api."""

import json
import re
from datetime import UTC, datetime
from pathlib import Path
from unittest.mock import patch

import pytest
from aioresponses import aioresponses

from myskoda.models.common import OpenState
from myskoda.models.departure import DepartureInfo
from myskoda.models.status import DoorWindowState
from myskoda.models.trip_statistics import VehicleType
from myskoda.myskoda import MySkoda
from myskoda.utils import to_iso8601

FIXTURES_DIR = Path(__file__).parent.joinpath("fixtures")

print(f"__file__ = {__file__}")


@pytest.fixture(name="vehicle_infos")
def load_vehicle_info() -> list[str]:
    """Load vehicle-info fixture."""
    vehicle_infos = []
    # TODO @dvx76: probably just glob all files
    for path in [
        "enyaq/garage_vehicles_iv80.json",
        "enyaq/garage_vehicles_iv80_coupe.json",
        "superb/garage_vehicles_LK_liftback.json",
        "superb/garage_with_429_error.json",
    ]:
        with FIXTURES_DIR.joinpath(path).open() as file:
            vehicle_infos.append(file.read())
    return vehicle_infos


@pytest.mark.asyncio
async def test_get_info(
    vehicle_infos: list[str], myskoda: MySkoda, responses: aioresponses
) -> None:
    """Example unit test for RestAPI.get_info(). Needs more work."""
    for vehicle_info in vehicle_infos:
        vehicle_info_json = json.loads(vehicle_info)

        responses.get(
            url="https://mysmob.api.connect.skoda-auto.cz/api/v2/garage/vehicles/TMBJM0CKV1N12345"
            "?connectivityGenerations=MOD1&connectivityGenerations=MOD2&connectivityGenerations=MOD3"
            "&connectivityGenerations=MOD4",
            body=vehicle_info,
        )

        get_info_result = await myskoda.get_info(vehicle_info_json["vin"])

        # Should probably assert the whole thing. Just an example.
        assert get_info_result.name == vehicle_info_json["name"]


@pytest.fixture(name="vehicle_statuses")
def load_vehicle_status() -> list[str]:
    """Load vehicle-status fixture."""
    vehicle_statuses = []
    for path in [
        "superb/vehicle-status-doors-closed.json",
        "superb/vehicle-status-right-front-door-opened.json",
        "superb/vehicle-status-left-back-door-trunk-bonnet-opened.json",
    ]:
        with FIXTURES_DIR.joinpath(path).open() as file:
            vehicle_statuses.append(file.read())
    return vehicle_statuses


@pytest.mark.asyncio
async def test_get_status(
    vehicle_statuses: list[str], myskoda: MySkoda, responses: aioresponses
) -> None:
    """Example unit test for RestAPI.get_status(). Needs more work."""
    for vehicle_status in vehicle_statuses:
        vehicle_status_json = json.loads(vehicle_status)

        target_vin = "TMBJM0CKV1N12345"

        responses.get(
            url=f"https://mysmob.api.connect.skoda-auto.cz/api/v2/vehicle-status/{target_vin}",
            body=vehicle_status,
        )
        get_status_result = await myskoda.get_status(target_vin)

        assert get_status_result.overall.lights == vehicle_status_json["overall"]["lights"]
        assert get_status_result.overall.doors == vehicle_status_json["overall"]["doors"]
        assert get_status_result.detail.bonnet == vehicle_status_json["detail"]["bonnet"]
        assert get_status_result.detail.trunk == vehicle_status_json["detail"]["trunk"]


@pytest.fixture(name="air_conditioning")
def load_air_conditioning() -> list[str]:
    """Load air-conditioning fixture."""
    air_conditioning = []
    for path in [
        "enyaq/air-conditioning-heating.json",
        "enyaq/air-conditioning-no-steering.json",
        "other/air-conditioning-idle.json",
        "superb/air-conditioning-aux-heater.json",
        "superb/air-conditioning-idle.json",
    ]:
        with FIXTURES_DIR.joinpath(path).open() as file:
            air_conditioning.append(file.read())
    return air_conditioning


@pytest.mark.asyncio
async def test_get_air_conditioning(
    air_conditioning: list[str], myskoda: MySkoda, responses: aioresponses
) -> None:
    """Example unit test for RestAPI.get_air_conditioning(). Needs more work."""
    for air_conditioning_status in air_conditioning:
        air_conditioning_status_json = json.loads(air_conditioning_status)

        target_vin = "TMBJM0CKV1N12345"
        responses.get(
            url=f"https://mysmob.api.connect.skoda-auto.cz/api/v2/air-conditioning/{target_vin}",
            body=air_conditioning_status,
        )
        get_status_result = await myskoda.get_air_conditioning(target_vin)

        assert get_status_result.state == air_conditioning_status_json["state"]
        assert (
            get_status_result.window_heating_state is None
            or get_status_result.window_heating_state.front
            == air_conditioning_status_json["windowHeatingState"]["front"]
        )


@pytest.fixture(name="auxiliary_heating")
def load_auxiliary_heating() -> list[str]:
    """Load auxiliary_heating fixture."""
    auxiliary_heating = []
    for path in [
        "other/auxiliary-heating-idle.json",
    ]:
        with FIXTURES_DIR.joinpath(path).open() as file:
            auxiliary_heating.append(file.read())
    return auxiliary_heating


@pytest.mark.asyncio
async def test_get_auxiliary_heating(
    auxiliary_heating: list[str], myskoda: MySkoda, responses: aioresponses
) -> None:
    """Example unit test for RestAPI.get_auxiliary_heating(). Needs more work."""
    for auxiliary_heating_status in auxiliary_heating:
        auxiliary_heating_status_json = json.loads(auxiliary_heating_status)

        target_vin = "TMBJM0CKV1N12345"
        responses.get(
            url=f"https://mysmob.api.connect.skoda-auto.cz/api/v2/air-conditioning/{target_vin}/auxiliary-heating",
            body=auxiliary_heating_status,
        )
        get_status_result = await myskoda.get_auxiliary_heating(target_vin)

        assert get_status_result.state == auxiliary_heating_status_json["state"]


@pytest.mark.asyncio
async def test_get_computed_status(myskoda: MySkoda, responses: aioresponses) -> None:
    """Test case for computed values of doors and windows state."""
    file_name = "superb/vehicle-status-left-back-door-trunk-bonnet-opened.json"
    vehicle_status = FIXTURES_DIR.joinpath(file_name).read_text()

    target_vin = "TMBJM0CKV1N12345"
    responses.get(
        url=f"https://mysmob.api.connect.skoda-auto.cz/api/v2/vehicle-status/{target_vin}",
        body=vehicle_status,
    )
    get_status_result = await myskoda.get_status(target_vin)

    assert get_status_result.left_front_door == DoorWindowState.CLOSED
    assert get_status_result.left_back_door == DoorWindowState.DOOR_OPEN
    assert get_status_result.right_front_door == DoorWindowState.CLOSED
    assert get_status_result.right_back_door == DoorWindowState.WINDOW_OPEN
    assert get_status_result.detail.bonnet == OpenState.OPEN
    assert get_status_result.detail.trunk == OpenState.OPEN


@pytest.fixture(name="charging")
def load_charging() -> list[str]:
    """Load charging fixture."""
    charging = []
    for path in [
        "superb/charging-iV.json",
    ]:
        with FIXTURES_DIR.joinpath(path).open() as file:
            charging.append(file.read())
    return charging


@pytest.mark.asyncio
async def test_charging(charging: list[str], myskoda: MySkoda, responses: aioresponses) -> None:
    """Example unit test for RestAPI.charging(). Needs more work."""
    for charging_status in charging:
        air_conditioning_status_json = json.loads(charging_status)

        target_vin = "TMBJM0CKV1N12345"
        responses.get(
            url=f"https://mysmob.api.connect.skoda-auto.cz/api/v1/charging/{target_vin}",
            body=charging_status,
        )
        get_status_result = await myskoda.get_charging(target_vin)

        assert get_status_result.status is not None
        assert get_status_result.status.state == air_conditioning_status_json["status"]["state"]
        assert (
            get_status_result.settings.max_charge_current_ac
            == air_conditioning_status_json["settings"]["maxChargeCurrentAc"]
        )


@pytest.fixture(name="charging_profiles")
def load_chargingprofiles() -> list[str]:
    """Load charging profile fixture."""
    charging_profiles = []
    for path in [
        "enyaq/charging-profiles.json",
    ]:
        with FIXTURES_DIR.joinpath(path).open() as file:
            charging_profiles.append(file.read())
    return charging_profiles


@pytest.mark.asyncio
async def test_charging_profiles(
    charging_profiles: list[str], myskoda: MySkoda, responses: aioresponses
) -> None:
    for charging_profile in charging_profiles:
        charging_profile_status_json = json.loads(charging_profile)

        target_vin = "TMBJM0CKV1N12345"
        responses.get(
            url=f"https://mysmob.api.connect.skoda-auto.cz/api/v1/charging/{target_vin}/profiles",
            body=charging_profile,
        )
        get_status_result = await myskoda.get_charging_profiles(target_vin)

        assert get_status_result.charging_profiles is not None
        assert (
            get_status_result.charging_profiles[0].id
            == charging_profile_status_json["chargingProfiles"][0]["id"]
        )


@pytest.fixture(name="trip_statistics")
def load_trip_statistics() -> list[str]:
    """Load trip statistics fixture."""
    trip_statistics = []
    for path in [
        "superb/trip-statistics-iV.json",
    ]:
        with FIXTURES_DIR.joinpath(path).open() as file:
            trip_statistics.append(file.read())
    return trip_statistics


@pytest.mark.asyncio
async def test_trip_statistics(
    trip_statistics: list[str], myskoda: MySkoda, responses: aioresponses
) -> None:
    """Example unit test for RestAPI.trip_statistics(). Needs more work."""
    for trip_statistics_input in trip_statistics:
        trip_statistics_json = json.loads(trip_statistics_input)

        target_vin = "TMBJM0CKV1N12345"
        responses.get(
            url=f"https://mysmob.api.connect.skoda-auto.cz/api/v1/trip-statistics/{target_vin}"
            "?offsetType=week&offset=0&timezone=Europe%2FBerlin",
            body=trip_statistics_input,
        )
        get_status_result = await myskoda.get_trip_statistics(target_vin)

        assert (
            get_status_result.overall_average_travel_time_in_min
            == trip_statistics_json["overallAverageTravelTimeInMin"]
        )
        assert get_status_result.vehicle_type == VehicleType.HYBRID


@pytest.fixture(name="vehicle_connection_statuses")
def load_vehicle_connection_status() -> list[str]:
    """Load connection status fixture."""
    vehicle_connection_statuses = []
    for path in [
        "other/vehicle-connection-status.json",
    ]:
        with FIXTURES_DIR.joinpath(path).open() as file:
            vehicle_connection_statuses.append(file.read())
    return vehicle_connection_statuses


@pytest.mark.asyncio
async def test_vehicle_connection_status(
    vehicle_connection_statuses: list[str], myskoda: MySkoda, responses: aioresponses
) -> None:
    """Unit test for RestAPI.get_connection_status(). Needs more work."""
    for connection_status in vehicle_connection_statuses:
        connection_status_json = json.loads(connection_status)

        target_vin = "TMBJM0CKV1N12345"
        responses.get(
            url=f"https://mysmob.api.connect.skoda-auto.cz/api/v2/connection-status/{target_vin}/readiness",
            body=connection_status,
        )
        get_connection_status = await myskoda.get_connection_status(target_vin)

        assert get_connection_status.unreachable == connection_status_json["unreachable"]
        assert get_connection_status.in_motion == connection_status_json["inMotion"]
        assert (
            get_connection_status.battery_protection_limit_on
            == connection_status_json["batteryProtectionLimitOn"]
        )


@pytest.fixture(name="charging_histories")
def load_charging_histories() -> list[str]:
    """Load charging history fixtures."""
    charging_histories = []
    for path in [
        "other/charging-history.json",
    ]:
        with FIXTURES_DIR.joinpath(path).open() as file:
            charging_histories.append(file.read())
    return charging_histories


@pytest.mark.asyncio
async def test_charging_history(
    charging_histories: list[str], myskoda: MySkoda, responses: aioresponses
) -> None:
    """Unit test for RestAPI.get_charging_history(). Needs more work."""
    for charging_history in charging_histories:
        charging_history_json = json.loads(charging_history)

        target_vin = "TMBJM0CKV1N12345"
        request_limit: int = 50
        responses.get(
            url=f"https://mysmob.api.connect.skoda-auto.cz/api/v1/charging/{target_vin}/history?userTimezone=UTC&limit={request_limit}",
            body=charging_history,
        )

        get_charging_history = await myskoda.get_charging_history(target_vin)
        # Make sure the cursor is correct
        if get_charging_history.next_cursor:
            assert (
                to_iso8601(get_charging_history.next_cursor) == charging_history_json["nextCursor"]
            )
        # Make sure we dont get more than we asked for
        assert (
            len([session for period in get_charging_history.periods for session in period.sessions])
            < request_limit + 1
        )
        assert len(get_charging_history.periods) > 0


@pytest.fixture(name="spin_statuses")
def load_spin_status() -> list[str]:
    """Load spin-status fixture."""
    spin_statuses = []
    for path in [
        "superb/verify-spin-correct.json",
        "superb/verify-spin-incorrect.json",
    ]:
        with FIXTURES_DIR.joinpath(path).open() as file:
            spin_statuses.append(file.read())
    return spin_statuses


@pytest.mark.asyncio
@pytest.mark.parametrize("spin", ["1234"])
async def test_get_spin_status(
    spin_statuses: list[str], myskoda: MySkoda, responses: aioresponses, spin: str
) -> None:
    """Example unit test for RestAPI.get_status(). Needs more work."""
    for spin_status in spin_statuses:
        spin_status_json = json.loads(spin_status)

        responses.post(
            url="https://mysmob.api.connect.skoda-auto.cz/api/v1/spin/verify",
            body=spin_status,
        )
        get_spin_status_result = await myskoda.verify_spin(spin)

        assert get_spin_status_result.verification_status == spin_status_json["verificationStatus"]
        if get_spin_status_result.spin_status is not None:
            assert (
                get_spin_status_result.spin_status.remaining_tries
                == spin_status_json["spinStatus"]["remainingTries"]
            )
            assert (
                get_spin_status_result.spin_status.locked_waiting_time_in_seconds
                == spin_status_json["spinStatus"]["lockedWaitingTimeInSeconds"]
            )
            assert (
                get_spin_status_result.spin_status.state == spin_status_json["spinStatus"]["state"]
            )


@pytest.fixture(name="departure_timers")
def load_departure_timers() -> list[str]:
    """Load departure timers fixture."""
    departure_timers = []
    for path in [
        "other/departure-timers.json",
    ]:
        with FIXTURES_DIR.joinpath(path).open() as file:
            departure_timers.append(file.read())
    return departure_timers


@pytest.mark.asyncio
async def test_get_departure_timers(
    departure_timers: list[str], myskoda: MySkoda, responses: aioresponses
) -> None:
    """Example unit test for RestAPI.charging(). Needs more work."""
    for departure_timer in departure_timers:
        target_vin = "TMBJM0CKV1N12345"
        base_url = f"https://mysmob.api.connect.skoda-auto.cz/api/v1/vehicle-automatization/{target_vin}/departure/timers"
        # Add a regular expression for the dynamic timestamp query parameter
        url_pattern = re.compile(rf"{base_url}\?deviceDateTime=.*")

        responses.get(
            url=url_pattern,
            body=departure_timer,
        )

        with patch("myskoda.models.common.datetime") as mock_datetime:
            mock_datetime.now.return_value = datetime(2024, 1, 1, 0, 0, 0, tzinfo=UTC)
            get_departure_timers_result = await myskoda.get_departure_timers(target_vin)

            assert get_departure_timers_result == DepartureInfo.from_json(departure_timer)



"""Unit tests for generated fixtures."""

import re
from pathlib import Path

import pytest
from aioresponses import aioresponses

from myskoda.anonymize import VIN
from myskoda.const import BASE_URL_SKODA
from myskoda.models.fixtures import Fixture, FixtureReportGet
from myskoda.myskoda import MySkoda

FIXTURES_DIR = Path(__file__).parent.parent / "fixtures"


@pytest.mark.asyncio
async def test_report_get(
    report: FixtureReportGet, responses: aioresponses, myskoda: MySkoda
) -> None:
    # Check if the URL contains a query string
    if report.url and "?" in report.url:
        url_pattern = re.compile(rf"{BASE_URL_SKODA}/api{report.url.split('?')[0]}\?.*")
    else:
        url_pattern = re.compile(rf"{BASE_URL_SKODA}/api{report.url}")
    responses.get(url=url_pattern, body=report.raw)

    result = await myskoda.get_endpoint(VIN, report.endpoint, anonymize=True)
    result = result.result.to_dict()

    # Remove timestamp
    result["timestamp"] = None
    if (res := report.result) is not None:
        if "timestamp" in res:
            res["timestamp"] = None
        mr = res.get("maintenance_report")
        if mr:
            mr["timestamp"] = None
            result["maintenance_report"]["timestamp"] = None

    assert result == report.result


def pytest_generate_tests(metafunc: pytest.Metafunc) -> None:
    parameters = []

    for file in FIXTURES_DIR.glob("**/*.yaml"):
        text = file.read_text(encoding="utf-8")
        fixture = Fixture.from_yaml(text)
        if fixture.reports is None:
            continue
        for report in fixture.reports:
            if not report.success:
                continue
            parameters.append(report)

    metafunc.parametrize("report", parameters)



"""Unit tests for myskoda.driving_range."""

from pathlib import Path

import pytest
from aioresponses import aioresponses

from myskoda.models.driving_range import EngineType
from myskoda.myskoda import MySkoda

FIXTURES_DIR = Path(__file__).parent.joinpath("fixtures")


print(f"__file__ = {__file__}")


@pytest.mark.asyncio
@pytest.mark.parametrize(
    ("fixture_file", "expected_values"),
    [
        (
            "superb/driving-range-car-type-hybrid.json",
            {
                "car_type": EngineType.HYBRID,
                "primary_engine": EngineType.GASOLINE,
                "expected_primary_range": 670,
                "secondary_engine": EngineType.ELECTRIC,
                "expected_secondary_range": 7,
            },
        ),
        (
            "enyaq/driving-range-iv80-car-type-electric.json",
            {
                "car_type": EngineType.ELECTRIC,
                "primary_engine": EngineType.ELECTRIC,
                "expected_primary_range": 139,
            },
        ),
    ],
)
async def test_get_driving_range(
    fixture_file: str,
    expected_values: dict[str, dict[str, EngineType | int]],
    myskoda: MySkoda,
    responses: aioresponses,
) -> None:
    """Test case for driving range response."""
    vehicle_status = FIXTURES_DIR.joinpath(fixture_file).read_text()
    target_vin = "TMBJM0CKV1N12345"
    responses.get(
        url=f"https://mysmob.api.connect.skoda-auto.cz/api/v2/vehicle-status/{target_vin}/driving-range",
        body=vehicle_status,
    )

    get_status_result = await myskoda.get_driving_range(target_vin)

    assert get_status_result.car_type == expected_values["car_type"]
    assert get_status_result.primary_engine_range.engine_type == expected_values["primary_engine"]
    assert (
        get_status_result.primary_engine_range.remaining_range_in_km
        == expected_values["expected_primary_range"]
    )
    if get_status_result.secondary_engine_range is not None:
        assert (
            get_status_result.secondary_engine_range.engine_type
            == expected_values["secondary_engine"]
        )
        assert (
            get_status_result.secondary_engine_range.remaining_range_in_km
            == expected_values["expected_secondary_range"]
        )



"""CLI commands for dealing with MQTT."""

import asyncio
from typing import TYPE_CHECKING

import asyncclick as click
from asyncclick.core import Context
from termcolor import colored

from myskoda.cli.utils import mqtt_required
from myskoda.models.event import BaseEvent, OperationName
from myskoda.myskoda import MqttDisabledError

if TYPE_CHECKING:
    from myskoda import MySkoda


@click.command()
@click.argument("operation", type=click.Choice(OperationName))  # pyright: ignore [reportArgumentType]
@click.pass_context
@mqtt_required
async def wait_for_operation(ctx: Context, operation: OperationName) -> None:
    """Wait for the operation with the specified name to complete."""
    myskoda: MySkoda = ctx.obj["myskoda"]
    if myskoda.mqtt is None:
        raise MqttDisabledError

    print(f"Waiting for an operation {colored(operation, 'green')} to start and complete...")

    await myskoda.mqtt.wait_for_operation(operation)
    print("Completed.")


@click.command()
@click.pass_context
@mqtt_required
async def subscribe(ctx: Context) -> None:
    """Connect to the MQTT broker and listen for messages."""
    myskoda: MySkoda = ctx.obj["myskoda"]

    async def on_event(event: BaseEvent) -> None:
        ctx.obj["print"](event.to_dict())

    myskoda.subscribe_events(on_event)
    await asyncio.Event().wait()



"""Represents a whole vehicle."""

from .models.air_conditioning import AirConditioning
from .models.auxiliary_heating import AuxiliaryHeating
from .models.charging import Charging
from .models.departure import DepartureInfo
from .models.driving_range import DrivingRange
from .models.health import Health
from .models.info import CapabilityId, Info
from .models.maintenance import Maintenance
from .models.position import ParkingPositionV3, Positions
from .models.status import Status
from .models.trip_statistics import TripStatistics
from .models.vehicle_connection_status import VehicleConnectionStatus


class Vehicle:
    """Main model for a Vehicle. Holds all Vehicle information."""

    info: Info
    charging: Charging | None = None
    status: Status | None = None
    air_conditioning: AirConditioning | None = None
    auxiliary_heating: AuxiliaryHeating | None = None
    positions: Positions | None = None
    parking_position: ParkingPositionV3 | None = None
    driving_range: DrivingRange | None = None
    trip_statistics: TripStatistics | None = None
    maintenance: Maintenance
    health: Health | None = None
    departure_info: DepartureInfo | None = None
    connection_status: VehicleConnectionStatus | None = None

    def __init__(self, info: Info, maintenance: Maintenance) -> None:  # pragma: no cover
        self.info = info
        self.maintenance = maintenance

    def has_capability(self, cap: CapabilityId) -> bool:
        """Check for a capability.

        Checks whether a vehicle generally has a capability.
        Does not check whether it's actually available.
        """
        return self.info.has_capability(cap)

    def is_capability_available(self, cap: CapabilityId) -> bool:
        """Check for capability availability.

        Checks whether the vehicle has the capability and whether it is currently
        available. A capability can be unavailable for example if it's deactivated
        by the currently active user.
        """
        return self.info.is_capability_available(cap)



"""Models related to service events from the MQTT broker.

Service Event example:
{
    "version": 1,
    "traceId": "4a13b906-e13d-4ea5-a377-6cb70f790337",
    "producer": "SKODA_MHUB",
    "name": "change-soc",
    "timestamp": "2025-05-11T07:35:18Z",
    "data": {
        "mode": "manual",
        "state": "charging",
        "soc": "79",
        "chargedRange": "355",
        "timeToFinish": "130",
        "userId": "b8bc126c-ee36-402b-8723-2c1c3dff8dec",
        "vin": "TMOCKAA0AA000000",
    },
}
"""

from dataclasses import dataclass, field
from enum import StrEnum

from mashumaro import field_options
from mashumaro.types import Discriminator

from myskoda.models.charging import ChargeMode, ChargingState

from .base import BaseEvent, BaseEventData, EventType


class UnexpectedChargeModeError(Exception):
    pass


class UnexpectedChargingStateError(Exception):
    pass


class ServiceEventName(StrEnum):
    """List of known Service Event Names."""

    CHANGE_ACCESS = "change-access"
    CHANGE_CHARGE_MODE = "change-charge-mode"
    CHANGE_LIGHTS = "change-lights"
    CHANGE_ODOMETER = "change-odometer"
    CHANGE_REMAINING_TIME = "change-remaining-time"
    CHANGE_SOC = "change-soc"
    CHARGING_COMPLETED = "charging-completed"
    CHARGING_ERROR = "charging-error"
    CHARGING_STATUS_CHANGED = "charging-status-changed"
    CLIMATISATION_COMPLETED = "climatisation-completed"
    DEPARTURE_READY = "departure-ready"
    DEPARTURE_STATUS_CHANGED = "departure-status-changed"
    DEPARTURE_ERROR_PLUG = "departure-error-plug"


class ServiceEventError(StrEnum):
    STOPPED_DEVICE = "STOPPED_DEVICE"
    CLIMA = "CLIMA"


def _deserialize_time_to_finish(value: int | str) -> int | None:
    if value == "null":
        return None
    return int(value)


@dataclass(frozen=True)
class ServiceEventData(BaseEventData):
    """Base model for data in all Service Events."""


@dataclass(frozen=True)
class ServiceEventErrorData(ServiceEventData):
    """Model for service event data with an error."""

    error_code: ServiceEventError = field(metadata=field_options(alias="errorCode"))


@dataclass(frozen=True)
class ServiceEvent(BaseEvent):
    """Base model for all Service Events.

    Service Events are unsolicited events emitted by the MQTT bus towards the client.
    Service Events have a 'name' field which can be used as a discriminator.
    """

    class Config(BaseEvent.Config):  # noqa: D106
        discriminator = Discriminator(field="name", include_subtypes=True)

    event_type = EventType.SERVICE_EVENT
    producer: str
    name: ServiceEventName
    data: ServiceEventData


# ChargeMode and ChargingState use different values in API responses and MQTT events.
# This allows using the same Enum for both.
ChargeMode.HOME_STORAGE_CHARGING._add_value_alias_("homeStorageCharging")  # type: ignore[reportAttributeAccessIssue]
ChargeMode.IMMEDIATE_DISCHARGING._add_value_alias_("immediateDischarging")  # type: ignore[reportAttributeAccessIssue]
ChargeMode.ONLY_OWN_CURRENT._add_value_alias_("onlyOwnCurrent")  # type: ignore[reportAttributeAccessIssue]
ChargeMode.PREFERRED_CHARGING_TIMES._add_value_alias_("preferredChargingTimes")  # type: ignore[reportAttributeAccessIssue]
ChargeMode.TIMER_CHARGING_WITH_CLIMATISATION._add_value_alias_("timerChargingWithClimatisation")  # type: ignore[reportAttributeAccessIssue]
ChargingState.READY_FOR_CHARGING._add_value_alias_("chargePurposeReachedAndNotConservationCharging")  # type: ignore[reportAttributeAccessIssue]
ChargingState.READY_FOR_CHARGING._add_value_alias_("readyForCharging")  # type: ignore[reportAttributeAccessIssue]
ChargingState.CONNECT_CABLE._add_value_alias_("notReadyForCharging")  # type: ignore[reportAttributeAccessIssue]
ChargingState.CONSERVING._add_value_alias_("chargePurposeReachedAndConservation")  # type: ignore[reportAttributeAccessIssue]


@dataclass(frozen=True)
class ServiceEventChangeSocData(ServiceEventData):
    """Charging data inside charging service event change-soc.

    TODO: Remove the None defaults where they aren't really needed...
    """

    mode: ChargeMode | None = None
    state: ChargingState | None = None
    soc: int | None = field(default=None)
    charged_range: int | None = field(default=None, metadata=field_options(alias="chargedRange"))
    time_to_finish: int | None = field(
        default=None,
        metadata=field_options(alias="timeToFinish", deserialize=_deserialize_time_to_finish),
    )


class ServiceEventAirConditioning(ServiceEvent):
    """Group events under topic 'service-event/air-conditioning'."""


@dataclass(frozen=True)
class ServiceEventClimatisationCompleted(ServiceEventAirConditioning):
    name = ServiceEventName.CLIMATISATION_COMPLETED


class ServiceEventCharging(ServiceEvent):
    """Group events under topic 'service-event/charging'."""


@dataclass(frozen=True)
class ServiceEventChangeChargeMode(ServiceEventCharging):
    name = ServiceEventName.CHANGE_CHARGE_MODE


@dataclass(frozen=True)
class ServiceEventChangeRemainingTime(ServiceEventCharging):
    name = ServiceEventName.CHANGE_REMAINING_TIME


@dataclass(frozen=True)
class ServiceEventChangeSoc(ServiceEventCharging):
    data: ServiceEventChangeSocData
    name = ServiceEventName.CHANGE_SOC


@dataclass(frozen=True)
class ServiceEventChargingCompleted(ServiceEventCharging):
    data: ServiceEventChangeSocData
    name = ServiceEventName.CHARGING_COMPLETED


@dataclass(frozen=True)
class ServiceEventChargingError(ServiceEventCharging):
    data: ServiceEventErrorData
    name = ServiceEventName.CHARGING_ERROR


@dataclass(frozen=True)
class ServiceEventChargingStatusChanged(ServiceEventCharging):
    name = ServiceEventName.CHARGING_STATUS_CHANGED


class ServiceEventAccess(ServiceEvent):
    """Group events under topic 'service-event/vehicle-status/access'."""


@dataclass(frozen=True)
class ServiceEventChangeAccess(ServiceEventAccess):
    name = ServiceEventName.CHANGE_ACCESS


class ServiceEventLights(ServiceEvent):
    """Group events under topic 'service-event/vehicle-status/lights'."""


@dataclass(frozen=True)
class ServiceEventChangeLights(ServiceEventLights):
    name = ServiceEventName.CHANGE_LIGHTS


class ServiceEventOdometer(ServiceEvent):
    """Group events under topic 'service-event/vehicle-status/odometer'."""


@dataclass(frozen=True)
class ServiceEventChangeOdometer(ServiceEventOdometer):
    name = ServiceEventName.CHANGE_ODOMETER


class ServiceEventDeparture(ServiceEvent):
    """Group events under topic 'service-event/departure'."""


@dataclass(frozen=True)
class ServiceEventDepartureReady(ServiceEventDeparture):
    name = ServiceEventName.DEPARTURE_READY


@dataclass(frozen=True)
class ServiceEventDepartureStatusChanged(ServiceEventDeparture):
    name = ServiceEventName.DEPARTURE_STATUS_CHANGED


@dataclass(frozen=True)
class ServiceEventDepartureErrorPlug(ServiceEventDeparture):
    data: ServiceEventErrorData
    name = ServiceEventName.DEPARTURE_ERROR_PLUG



"""Utilities."""

import asyncio
import functools
from collections.abc import Awaitable, Callable
from datetime import UTC, datetime
from typing import ParamSpec

from .const import DEFAULT_DEBOUNCE_WAIT_SECONDS

# TODO @dvx76: Switch to Python 3.13 generics syntax once we're on 3.13
# https://docs.python.org/3/library/typing.html#typing.ParamSpec

P = ParamSpec("P")  # Represents the function parameters


def async_debounce(
    wait: float = DEFAULT_DEBOUNCE_WAIT_SECONDS, immediate: bool = False, queue: bool = True
) -> Callable[[Callable[P, Awaitable[object]]], Callable[P, Awaitable[None]]]:
    """Debounce decorator for async functions.

    NOTE: only works for functions returning None!

    Ensures that the decorated function is only executed once after the specified wait time,
    resetting if called again.

    When 'immediate' is True the first call is executed immediatally.
    When queue is True subsequent calls are still debounced normally. 'queue' does nothing when
    'immediate' is False.
    """
    task: asyncio.Task | None = None
    last_execution_time: float = 0.0

    def decorator(func: Callable[P, Awaitable[object]]) -> Callable[P, Awaitable[None]]:
        @functools.wraps(func)
        async def wrapper(*args: P.args, **kwargs: P.kwargs) -> None:
            nonlocal task, last_execution_time

            async def delayed_execution() -> object:
                nonlocal last_execution_time
                await asyncio.sleep(wait)
                last_execution_time = now
                return await func(*args, **kwargs)

            now = asyncio.get_running_loop().time()

            if immediate and now - last_execution_time >= wait:
                last_execution_time = now
                await func(*args, **kwargs)
                return

            if task:
                task.cancel()

            if not immediate or queue:
                task = asyncio.create_task(delayed_execution())

        return wrapper

    return decorator


def to_iso8601(dt: datetime) -> str:
    """Convert a datetime object to an ISO 8601 string.

    - Adds 'Z' if datetime is UTC.
    - Converts naive datetimes to UTC before formatting.
    """
    if dt.tzinfo is None:
        # Assume naive datetimes are in UTC
        dt = dt.replace(tzinfo=UTC)
    dt_utc = dt.astimezone(UTC)
    return dt_utc.isoformat().replace("+00:00", "Z")



"""Generate a set of test fixtures from your garage."""

from pathlib import Path

import asyncclick as click
from asyncclick.core import Context

from myskoda.models.fixtures import Endpoint
from myskoda.myskoda import MySkoda

FIXTURES_DIR = Path(__file__).parent.parent.parent / "tests" / "fixtures" / "gen"


@click.group()
@click.option(
    "vehicle",
    "--vehicle",
    help="Generate fixtures for this vehicle only (vin). Set to 'all' for all vehicles.",
    required=True,
)
@click.option("name", "--name", help="Short name describing the vehicle's state.", required=True)
@click.option("description", "--description", help="A longer description.")
@click.option("file", "--file", help="Override default file name.")
@click.pass_context
async def gen_fixtures(
    ctx: Context, vehicle: str, name: str, description: str, file: str | None
) -> None:
    """Interact with the MySkoda API."""
    myskoda: MySkoda = ctx.obj["myskoda"]

    default_filename = Path("fixtures") / f"{name.replace(' ', '_').lower()}.yaml"

    ctx.obj["vins"] = await get_vin_list(myskoda, vehicle)
    ctx.obj["name"] = name
    ctx.obj["description"] = description
    ctx.obj["file"] = Path(file) if file is not None else default_filename


@gen_fixtures.command()
@click.argument("endpoint", type=click.Choice(Endpoint))  # pyright: ignore [reportArgumentType]
@click.pass_context
async def get(ctx: Context, endpoint: Endpoint) -> None:
    myskoda: MySkoda = ctx.obj["myskoda"]

    fixture = await myskoda.generate_get_fixture(
        name=ctx.obj["name"],
        description=ctx.obj["description"],
        vins=ctx.obj["vins"],
        endpoint=endpoint,
    )

    text = str(fixture.to_yaml())

    file: Path = ctx.obj["file"]
    file.parent.mkdir(parents=True, exist_ok=True)
    file.write_text(text)


async def get_vin_list(myskoda: MySkoda, vehicle: str) -> list[str]:
    if vehicle != "all":
        return [vehicle]
    return await myskoda.list_vehicle_vins()



"""Models related to service events from the MQTT broker."""

from dataclasses import dataclass
from enum import StrEnum

from .base import BaseEvent, EventType


class AccountEventTopic(StrEnum):
    ACCOUNT_PRIVACY = "ACCOUNT_PRIVACY"


@dataclass(frozen=True)
class AccountEvent(BaseEvent):
    """Base model for all Account Events.

    TODO: We need examples of account events!
    """

    event_type = EventType.ACCOUNT_EVENT



"""Unit tests for myskoda.models.status."""

import json
from pathlib import Path

import pytest

from myskoda.models.status import DoorWindowState, Status

FIXTURES_DIR = Path(__file__).parent.joinpath("fixtures")


@pytest.fixture(name="vehicle_status")
def load_vehicle_status() -> dict:
    """Load vehicle-status fixture."""
    vehicle_status = (FIXTURES_DIR / "superb/vehicle-status-doors-closed.json").read_text()
    return json.loads(vehicle_status)


def _get_config_with_lightmode_onex_url(full_config: dict, onex_url: str) -> dict:
    full_config["renders"]["lightMode"]["oneX"] = onex_url
    return full_config


def test_door_window_states_all_closed_parsed_ok(vehicle_status: dict) -> None:
    """Test various door and window states."""
    test_json = _get_config_with_lightmode_onex_url(
        vehicle_status,
        "https://mysmob.api.connect.skoda-auto.cz/api/v2/vehicle-status/render?carType=LIMOUSINE&"
        "vehicleState=1-1-1-1-0-0-1-1-2"
        "&lastModifiedAt=1723053261&dimension=1x&theme=LIGHT",
    )
    status = Status.from_dict(test_json)
    assert status.left_front_door == DoorWindowState.CLOSED
    assert status.right_front_door == DoorWindowState.CLOSED
    assert status.left_back_door == DoorWindowState.CLOSED
    assert status.right_back_door == DoorWindowState.CLOSED


def test_door_window_states_doors_and_windows_opened_ok(vehicle_status: dict) -> None:
    """Test various door and window states."""
    test_json = _get_config_with_lightmode_onex_url(
        vehicle_status,
        "https://mysmob.api.connect.skoda-auto.cz/api/v2/vehicle-status/render?carType=LIMOUSINE&"
        "vehicleState=1-2-3-1-0-0-1-1-2"
        "&lastModifiedAt=1723053261&dimension=1x&theme=LIGHT",
    )
    status = Status.from_dict(test_json)
    assert status.left_front_door == DoorWindowState.CLOSED
    assert status.right_front_door == DoorWindowState.WINDOW_OPEN
    assert status.left_back_door == DoorWindowState.DOOR_OPEN
    assert status.right_back_door == DoorWindowState.CLOSED


def test_door_window_states_doors_with_windows_opened_ok(vehicle_status: dict) -> None:
    """Test various door and window states."""
    test_json = _get_config_with_lightmode_onex_url(
        vehicle_status,
        "https://mysmob.api.connect.skoda-auto.cz/api/v2/vehicle-status/render?carType=LIMOUSINE&"
        "vehicleState=1-2-3-4-0-0-1-1-2"
        "&lastModifiedAt=1723053261&dimension=1x&theme=LIGHT",
    )
    status = Status.from_dict(test_json)
    assert status.left_front_door == DoorWindowState.CLOSED
    assert status.right_front_door == DoorWindowState.WINDOW_OPEN
    assert status.left_back_door == DoorWindowState.DOOR_OPEN
    assert status.right_back_door == DoorWindowState.ALL_OPEN


def test_door_window_states_url_parsing_fails(vehicle_status: dict) -> None:
    """Test various door and window states."""
    test_json = _get_config_with_lightmode_onex_url(
        vehicle_status,
        r"htmob.api.connect.skoda-auto.cz/api\v2\vehicle-status/render+carType=LIMOUSINE&"
        "vehicleState=1-2-3-1-0-0-1-1-2&lastModifiedAt=1723053261&dimension=1x&theme=LIGHT",
    )
    status = Status.from_dict(test_json)
    assert status.left_front_door == DoorWindowState.UNKNOWN
    assert status.right_front_door == DoorWindowState.UNKNOWN
    assert status.left_back_door == DoorWindowState.UNKNOWN
    assert status.right_back_door == DoorWindowState.UNKNOWN


def test_door_window_states_no_query(vehicle_status: dict) -> None:
    """Test various door and window states."""
    test_json = _get_config_with_lightmode_onex_url(
        vehicle_status,
        "https://mysmob.api.connect.skoda-auto.cz/api/v2/vehicle-status/render?carType=LIMOUSINE&"
        "lastModifiedAt=1723053261&dimension=1x&theme=LIGHT",
    )
    status = Status.from_dict(test_json)
    assert status.left_front_door == DoorWindowState.UNKNOWN
    assert status.right_front_door == DoorWindowState.UNKNOWN
    assert status.left_back_door == DoorWindowState.UNKNOWN
    assert status.right_back_door == DoorWindowState.UNKNOWN


def test_door_window_states_query_has_nonint_values(vehicle_status: dict) -> None:
    """Test various door and window states."""
    test_json = _get_config_with_lightmode_onex_url(
        vehicle_status,
        "https://mysmob.api.connect.skoda-auto.cz/api/v2/vehicle-status/render?carType=LIMOUSINE&"
        "vehicleState=1-1-1-g-0-0-1-1-2"
        "&lastModifiedAt=1723053261&dimension=1x&theme=LIGHT",
    )
    status = Status.from_dict(test_json)
    assert status.left_front_door == DoorWindowState.CLOSED
    assert status.right_front_door == DoorWindowState.CLOSED
    assert status.left_back_door == DoorWindowState.CLOSED
    assert status.right_back_door == DoorWindowState.UNKNOWN


def test_door_window_states_query_has_illegal_value(vehicle_status: dict) -> None:
    """Test various door and window states."""
    test_json = _get_config_with_lightmode_onex_url(
        vehicle_status,
        "https://mysmob.api.connect.skoda-auto.cz/api/v2/vehicle-status/render?carType=LIMOUSINE&"
        "vehicleState=1-3-1-5-0-0-1-1-2"
        "&lastModifiedAt=1723053261&dimension=1x&theme=LIGHT",
    )
    status = Status.from_dict(test_json)
    assert status.left_front_door == DoorWindowState.CLOSED
    assert status.right_front_door == DoorWindowState.DOOR_OPEN
    assert status.left_back_door == DoorWindowState.CLOSED
    assert status.right_back_door == DoorWindowState.UNKNOWN



"""Models for responses of api/v1/charging/vin/profiles endpoint."""

from dataclasses import dataclass, field
from datetime import datetime, time

from mashumaro import field_options
from mashumaro.mixins.orjson import DataClassORJSONMixin

from .air_conditioning import TimerMode
from .charging import MaxChargeCurrent, PlugUnlockMode
from .common import BaseResponse, Coordinates, Weekday


@dataclass
class ChargingTimes(DataClassORJSONMixin):
    """Times a charging profile can be active."""

    id: int
    enabled: bool
    start_time: time = field(metadata=field_options(alias="startTime"))
    end_time: time = field(metadata=field_options(alias="endTime"))


@dataclass
class MinBatterySOC(DataClassORJSONMixin):
    """Settings for minimal battery SOC."""

    minimum_battery_state_of_charge_in_percent: int = field(
        metadata=field_options(alias="minimumBatteryStateOfChargeInPercent")
    )


@dataclass
class ProfileSettings(DataClassORJSONMixin):
    """Settings for a Charging location/Profile."""

    max_charging_current: MaxChargeCurrent = field(
        metadata=field_options(alias="maxChargingCurrent")
    )
    min_battery_state_of_charge: MinBatterySOC = field(
        metadata=field_options(alias="minBatteryStateOfCharge")
    )
    target_state_of_charge_in_percent: int = field(
        metadata=field_options(alias="targetStateOfChargeInPercent")
    )
    auto_unlock_plug_when_charged: PlugUnlockMode = field(
        metadata=field_options(alias="autoUnlockPlugWhenCharged")
    )


@dataclass
class ChargingTimers(DataClassORJSONMixin):
    """Timers for a Charging location."""

    id: int
    enabled: bool
    time: time
    type: TimerMode
    recurring_on: list[Weekday] = field(metadata=field_options(alias="recurringOn"))


@dataclass
class ChargingProfile(DataClassORJSONMixin):
    """Charging profile definition."""

    id: int
    name: str
    settings: ProfileSettings
    preferred_charging_times: list[ChargingTimes] = field(
        metadata=field_options(alias="preferredChargingTimes")
    )
    timers: list[ChargingTimers]
    location: Coordinates | None = field(default=None)


@dataclass
class CurrentProfile(DataClassORJSONMixin):
    """Information on the currently active charging profile."""

    id: int
    name: str
    target_state_of_charge_in_percent: int = field(
        metadata=field_options(alias="targetStateOfChargeInPercent")
    )
    next_charging_time: time | None = field(
        default=None, metadata=field_options(alias="nextChargingTime")
    )


@dataclass
class ChargingProfiles(BaseResponse):
    """Information related to location bound charging settings for an EV."""

    charging_profiles: list[ChargingProfile] = field(
        metadata=field_options(alias="chargingProfiles")
    )
    current_vehicle_position_profile: CurrentProfile | None = field(
        default=None, metadata=field_options(alias="currentVehiclePositionProfile")
    )
    car_captured_timestamp: datetime | None = field(
        default=None, metadata=field_options(alias="carCapturedTimestamp")
    )
